@@ -1,3 +1,5 @@
+/* -*- mode: C; indent-tabs-mode: t; tab-width: 8; c-basic-offset: 2; -*- */
+
 /*
  * This file is part of Seed, the GObject Introspection<->Javascript bindings.
  *
@@ -12,12 +14,15 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with Seed.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) Robert Carr 2008 <carrr@rpi.edu>
+ * Copyright (C) Robert Carr 2009 <carrr@rpi.edu>
  */
 
 #include "seed-private.h"
 #include <sys/mman.h>
 
+typedef GObject *(*GObjectConstructCallback) (GType, guint,
+					      GObjectConstructParam *);
+
 JSClassRef seed_gtype_class;
 GIBaseInfo *objectclass_info = NULL;
 GIBaseInfo *paramspec_info = NULL;
@@ -41,13 +46,13 @@ static JSValueRef
 seed_property_method_invoked (JSContextRef ctx,
 			      JSObjectRef function,
 			      JSObjectRef thisObject,
-			      size_t argumentCount,
+			      gsize argumentCount,
 			      const JSValueRef arguments[],
 			      JSValueRef * exception)
 {
   GParamSpec *spec;
   GObjectClass *class;
-  int property_count;
+  guint property_count;
   JSValueRef newcount, oldcount;
 
   if (argumentCount != 1)
@@ -87,7 +92,7 @@ static JSValueRef
 seed_gsignal_method_invoked (JSContextRef ctx,
 			     JSObjectRef function,
 			     JSObjectRef thisObject,
-			     size_t argumentCount,
+			     gsize argumentCount,
 			     const JSValueRef arguments[],
 			     JSValueRef * exception)
 {
@@ -166,15 +171,14 @@ seed_gsignal_method_invoked (JSContextRef ctx,
       if (n_params > 0)
 	{
 	  guint i;
+	  JSValueRef ptype;
 
 	  param_types = g_new0 (GType, n_params);
 	  for (i = 0; i < n_params; i++)
 	    {
-	      JSValueRef ptype = JSObjectGetPropertyAtIndex (ctx,
-							     (JSObjectRef)
-							     jsparams,
-							     i,
-							     exception);
+	      ptype = JSObjectGetPropertyAtIndex (ctx,
+						  (JSObjectRef)
+						  jsparams, i, exception);
 
 	      param_types[i] = seed_value_to_int (ctx, ptype, exception);
 	    }
@@ -271,8 +275,8 @@ seed_get_class_info_for_type (GType type)
       if (object_info)
 	{
 	  return
-	    (GIBaseInfo *)g_object_info_get_class_struct
-	                  ((GIObjectInfo *)object_info);
+	    (GIBaseInfo *) g_object_info_get_class_struct
+	    ((GIObjectInfo *) object_info);
 	}
       g_base_info_unref (object_info);
     }
@@ -282,7 +286,7 @@ seed_get_class_info_for_type (GType type)
 static void
 seed_attach_methods_to_class_object (JSContextRef ctx,
 				     JSObjectRef object,
-				     JSValueRef *exception)
+				     JSValueRef * exception)
 {
   seed_create_function (ctx, "c_install_property",
 			&seed_property_method_invoked, object);
@@ -290,26 +294,16 @@ seed_attach_methods_to_class_object (JSContextRef ctx,
 			&seed_gsignal_method_invoked, object);
 }
 
-static GObject *
-seed_gtype_construct (GType type,
-		      guint n_construct_params,
-		      GObjectConstructParam *construct_params)
+static void
+seed_gtype_call_construct (GType type, GObject * object)
 {
   JSContextRef ctx;
   JSObjectRef func, this_object;
   JSValueRef exception = NULL, args[1];
-  GObject *object;
-  GType parent;
-  GObjectClass *parent_class;
-
-  parent = g_type_parent (type);
-  parent_class = g_type_class_ref (parent);
-
-  object = parent_class->constructor (type, n_construct_params, construct_params);
-
-  g_type_class_unref (parent_class);
+  gchar *mes;
 
   func = g_type_get_qdata (type, qiinit);
+
   if (func)
     {
       ctx = JSGlobalContextCreateInGroup (context_group, 0);
@@ -323,87 +317,135 @@ seed_gtype_construct (GType type,
       JSObjectCallAsFunction (ctx, func, this_object, 1, args, &exception);
       if (exception)
 	{
-	  gchar *mes = seed_exception_to_string (ctx, exception);
+	  mes = seed_exception_to_string (ctx, exception);
 	  g_warning ("Exception in instance construction. %s \n", mes);
 	  g_free (mes);
 	}
       JSGlobalContextRelease ((JSGlobalContextRef) ctx);
     }
 
+
+}
+
+static GObject *
+seed_gtype_construct (GType type,
+		      guint n_construct_params,
+		      GObjectConstructParam * construct_params)
+{
+  GObject *object;
+  GType parent;
+  GObjectClass *parent_class;
+
+  parent = g_type_parent (type);
+  parent_class = g_type_class_ref (parent);
+
+  if (parent_class->constructor == seed_gtype_construct)
+    {
+      GType t = parent;
+      parent = g_type_parent (parent);
+
+      g_type_class_unref (parent_class);
+      parent_class = g_type_class_ref (parent);
+
+      object =
+	parent_class->constructor (type, n_construct_params,
+				   construct_params);
+      
+      seed_gtype_call_construct (t, object);
+
+      g_type_class_unref (parent_class);
+    }
+  else
+    object =
+      parent_class->constructor (type, n_construct_params, construct_params);
+
+  seed_gtype_call_construct (type, object);
+
+  g_type_class_unref (parent_class);
+
   return object;
 }
 
 static void
 seed_gtype_install_signals (JSContextRef ctx,
-			    JSObjectRef definition,
-			    GType type)
+			    JSObjectRef definition, GType type,
+			    JSValueRef * exception)
 {
-  JSObjectRef signals;
+  JSObjectRef signals, signal_def;
   JSValueRef jslength;
-  guint i, length;
+  guint i, j, length;
+  GType return_type;
+  GType *param_types = NULL;
+  guint n_params = 0;
+  GSignalFlags flags;
+  JSValueRef jsname, jsflags, jsreturn_type, jsparams;
+  gchar *name;
 
-  signals = (JSObjectRef) seed_object_get_property (ctx, definition, "signals");
-  if (JSValueIsNull(ctx, signals) || !JSValueIsObject (ctx, signals))
+  signals =
+    (JSObjectRef) seed_object_get_property (ctx, definition, "signals");
+  if (JSValueIsNull (ctx, signals) || !JSValueIsObject (ctx, signals))
     return;
 
   jslength = seed_object_get_property (ctx, signals, "length");
   if (JSValueIsNull (ctx, jslength))
     return;
 
-  length = seed_value_to_uint (ctx, jslength, NULL);
+  length = seed_value_to_uint (ctx, jslength, exception);
   for (i = 0; i < length; i++)
     {
-      GType return_type;
-      GType *param_types = NULL;
-      guint n_params = 0;
-      GSignalFlags flags;
-      JSValueRef jsname, jsflags, jsreturn_type, jsparams;
-      gchar *name;
-      JSObjectRef signal_def = (JSObjectRef)JSObjectGetPropertyAtIndex (ctx,
-									(JSObjectRef) signals,
-									i,
-									NULL);
+      signal_def = (JSObjectRef) JSObjectGetPropertyAtIndex (ctx,
+							     (JSObjectRef)
+							     signals, i,
+							     exception);
 
-      if (JSValueIsNull (ctx, signal_def) || !JSValueIsObject(ctx, signal_def))
+      if (JSValueIsNull (ctx, signal_def)
+	  || !JSValueIsObject (ctx, signal_def))
 	continue;
 
       // TODO: Error checking
       jsname = seed_object_get_property (ctx, signal_def, "name");
-      name = seed_value_to_string (ctx, jsname, NULL);
+      name = seed_value_to_string (ctx, jsname, exception);
 
-      SEED_NOTE(GTYPE, "Installing signal with name: %s on type: %s",
-		name, g_type_name (type));
+      SEED_NOTE (GTYPE, "Installing signal with name: %s on type: %s",
+		 name, g_type_name (type));
 
-      jsflags = seed_object_get_property (ctx, (JSObjectRef) signal_def, "flags");
+      jsflags =
+	seed_object_get_property (ctx, (JSObjectRef) signal_def, "flags");
       if (JSValueIsNull (ctx, jsflags) || !JSValueIsNumber (ctx, jsflags))
 	flags = G_SIGNAL_RUN_LAST;
       else
-	flags = seed_value_to_long (ctx, jsflags, NULL);
+	flags = seed_value_to_long (ctx, jsflags, exception);
 
-      jsreturn_type = seed_object_get_property (ctx, signal_def, "return_type");
-      if (JSValueIsNull (ctx, jsreturn_type) || !JSValueIsNumber (ctx, jsreturn_type))
+      jsreturn_type =
+	seed_object_get_property (ctx, signal_def, "return_type");
+      if (JSValueIsNull (ctx, jsreturn_type)
+	  || !JSValueIsNumber (ctx, jsreturn_type))
 	return_type = G_TYPE_NONE;
       else
-	return_type = seed_value_to_long (ctx, jsreturn_type, NULL);
+	return_type = seed_value_to_long (ctx, jsreturn_type, exception);
 
       jsparams = seed_object_get_property (ctx, signal_def, "parameters");
 
       if (!JSValueIsNull (ctx, jsparams) && JSValueIsObject (ctx, jsparams))
 	{
-	  n_params = seed_value_to_int (ctx, seed_object_get_property (ctx, (JSObjectRef) jsparams, "length"), NULL);
+	  n_params =
+	    seed_value_to_int (ctx,
+			       seed_object_get_property (ctx,
+							 (JSObjectRef)
+							 jsparams, "length"),
+			       exception);
 	  if (n_params > 0)
 	    {
-	      guint i;
-
-	      param_types = g_alloca (sizeof (GType)*n_params);
-	      for (i = 0; i < n_params; i++)
+	      param_types = g_alloca (sizeof (GType) * n_params);
+	      for (j = 0; j < n_params; j++)
 		{
 		  JSValueRef ptype = JSObjectGetPropertyAtIndex (ctx,
 								 (JSObjectRef)
 								 jsparams,
-								 i,
-								 NULL);
-		  param_types[i] = (GType) seed_value_to_long (ctx, ptype, NULL);
+								 j,
+								 exception);
+		  param_types[j] =
+		    (GType) seed_value_to_long (ctx, ptype, exception);
 		}
 	    }
 	}
@@ -415,21 +457,332 @@ seed_gtype_install_signals (JSContextRef ctx,
       g_free (name);
 
     }
+}
 
+static unsigned int
+seed_gtype_install_properties (JSContextRef ctx,
+			       JSObjectRef definition,
+			       GType type, GObjectClass *g_class,
+			       JSValueRef * exception)
+{
+  JSObjectRef properties, property_def;
+  JSValueRef jslength;
+  guint i, length;
+  GType property_type, object_type;
+  GParamFlags flags;
+  JSValueRef jsname, jsflags, jsproperty_type, jsdefault_value, jsobject_type;
+  JSValueRef jsnick, jsblurb, jsmin_value, jsmax_value;
+  gchar *name , *nick, *blurb;
+  GParamSpec *pspec;
+  guint property_count = 0;
+
+  properties = (JSObjectRef) seed_object_get_property (ctx, definition,
+						       "properties");
+  if (JSValueIsNull (ctx, properties) || !JSValueIsObject (ctx, properties))
+    return 0;
+
+  jslength = seed_object_get_property (ctx, properties, "length");
+  if (JSValueIsNull (ctx, jslength))
+    return 0;
+
+  length = seed_value_to_uint (ctx, jslength, exception);
+  for (i = 0; i < length; i++)
+    {
+      property_def = (JSObjectRef) JSObjectGetPropertyAtIndex (ctx,
+							       (JSObjectRef)
+							       properties, i,
+							       NULL);
+
+      if (JSValueIsNull (ctx, property_def)
+	  || !JSValueIsObject (ctx, property_def))
+	continue;
+
+      jsname = seed_object_get_property (ctx, property_def, "name");
+      if (!JSValueIsString(ctx, jsname))
+	{
+	  seed_make_exception (ctx, exception, "PropertyInstallationError",
+			       "Property requires name attribute");
+	  return property_count;
+	}
+      name = seed_value_to_string (ctx, jsname, exception);
+
+      // Check for "nick" property; set to name if nonexistent
+      jsnick = seed_object_get_property (ctx, property_def, "nick");
+      if (!JSValueIsString(ctx, jsnick))
+	nick = name;
+      else
+	nick = seed_value_to_string (ctx, jsnick, exception);
+
+      // Check for "blurb" property; set to name if nonexistent
+      jsblurb = seed_object_get_property (ctx, property_def, "blurb");
+      if (!JSValueIsString(ctx, jsblurb))
+	blurb = name;
+      else
+	blurb = seed_value_to_string (ctx, jsblurb, exception);
+
+      SEED_NOTE (GTYPE, "Installing property with name: %s on type: %s",
+		 name, g_type_name (type));
+
+      // Flags default to read/write, non-construct
+      jsflags =
+	seed_object_get_property (ctx, (JSObjectRef) property_def, "flags");
+      if (JSValueIsNull (ctx, jsflags) || !JSValueIsNumber (ctx, jsflags))
+	flags = G_PARAM_READABLE | G_PARAM_WRITABLE;
+      else
+	flags = seed_value_to_long (ctx, jsflags, exception);
+
+      jsproperty_type = seed_object_get_property (ctx, property_def, "type");
+
+      if (JSValueIsNull (ctx, jsproperty_type) ||
+	  !JSValueIsNumber (ctx, jsproperty_type))
+	property_type = G_TYPE_NONE;
+      else
+	property_type = seed_value_to_long (ctx, jsproperty_type, exception);
+
+      jsdefault_value = seed_object_get_property (ctx, property_def,
+						  "default_value");
+
+      if (JSValueIsNull (ctx, jsdefault_value) ||
+	  JSValueIsUndefined (ctx, jsdefault_value))
+	{
+	  if(property_type == G_TYPE_OBJECT)
+	    continue;
+
+	  seed_make_exception (ctx, exception, "PropertyInstallationError",
+			       "Property of type %s requires default_value attribute",
+			       g_type_name(property_type));
+	  return property_count;
+	}
+
+      jsmin_value = seed_object_get_property (ctx, property_def,
+					      "minimum_value");
+      jsmax_value = seed_object_get_property (ctx, property_def,
+					      "maximum_value");
+
+      // Make sure min/max properties are defined, based on type
+      if(property_type == G_TYPE_CHAR ||
+	 property_type == G_TYPE_UCHAR ||
+	 property_type == G_TYPE_INT ||
+	 property_type == G_TYPE_UINT ||
+	 property_type == G_TYPE_INT64 ||
+	 property_type == G_TYPE_UINT64 ||
+	 property_type == G_TYPE_FLOAT ||
+	 property_type == G_TYPE_DOUBLE)
+	{
+	  if (JSValueIsNull (ctx, jsmin_value) ||
+	      !JSValueIsNumber (ctx, jsmin_value))
+	    {
+	      seed_make_exception (ctx, exception, "PropertyInstallationError",
+				   "Property of type %s requires minimum_value attribute",
+				   g_type_name(property_type));
+	      return property_count;
+	    }
+	  if (JSValueIsNull (ctx, jsmax_value) ||
+	      !JSValueIsNumber (ctx, jsmax_value))
+	    {
+	      seed_make_exception (ctx, exception, "PropertyInstallationError",
+				   "Property of type %s requires maximum_value attribute",
+				   g_type_name(property_type));
+	      return property_count;
+	    }
+	}
+
+
+      switch(property_type)
+	{
+	case G_TYPE_BOOLEAN:
+	  pspec = g_param_spec_boolean(name, nick, blurb,
+				       seed_value_to_boolean(ctx,
+							     jsdefault_value,
+							     exception), flags);
+	  break;
+	case G_TYPE_CHAR:
+	  pspec = g_param_spec_char(name, nick, blurb,
+				    seed_value_to_char(ctx,
+						       jsmin_value,
+						       exception),
+				    seed_value_to_char(ctx,
+						       jsmax_value,
+						       exception),
+				    seed_value_to_char(ctx,
+						       jsdefault_value,
+						       exception), flags);
+	  break;
+	case G_TYPE_UCHAR:
+	  pspec = g_param_spec_uchar(name, nick, blurb,
+				     seed_value_to_uchar(ctx,
+							 jsmin_value,
+							 exception),
+				     seed_value_to_uchar(ctx,
+							 jsmax_value,
+							 exception),
+				     seed_value_to_uchar(ctx,
+							 jsdefault_value,
+							 exception), flags);
+	  break;
+	case G_TYPE_INT:
+	  pspec = g_param_spec_int(name, nick, blurb,
+				   seed_value_to_int(ctx,
+						     jsmin_value,
+						     exception),
+				   seed_value_to_int(ctx,
+						     jsmax_value,
+						     exception),
+				   seed_value_to_int(ctx,
+						     jsdefault_value,
+						     exception), flags);
+	  break;
+	case G_TYPE_UINT:
+	  pspec = g_param_spec_uint(name, nick, blurb,
+				    seed_value_to_uint(ctx,
+						       jsmin_value,
+						       exception),
+				    seed_value_to_uint(ctx,
+						       jsmax_value,
+						       exception),
+				    seed_value_to_uint(ctx,
+						       jsdefault_value,
+						       exception), flags);
+	  break;
+	case G_TYPE_LONG:
+	  pspec = g_param_spec_long(name, nick, blurb,
+				    seed_value_to_long(ctx,
+						       jsmin_value,
+						       exception),
+				    seed_value_to_long(ctx,
+						       jsmax_value,
+						       exception),
+				    seed_value_to_long(ctx,
+						       jsdefault_value,
+						       exception), flags);
+	  break;
+	case G_TYPE_ULONG:
+	  pspec = g_param_spec_ulong(name, nick, blurb,
+				     seed_value_to_ulong(ctx,
+							 jsmin_value,
+							 exception),
+				     seed_value_to_ulong(ctx,
+							 jsmax_value,
+							 exception),
+				     seed_value_to_ulong(ctx,
+							 jsdefault_value,
+							 exception), flags);
+	  break;
+	case G_TYPE_INT64:
+	  pspec = g_param_spec_int64(name, nick, blurb,
+				     seed_value_to_int64(ctx,
+							 jsmin_value,
+							 exception),
+				     seed_value_to_int64(ctx,
+							 jsmax_value,
+							 exception),
+				     seed_value_to_int64(ctx,
+							 jsdefault_value,
+							 exception), flags);
+	  break;
+	case G_TYPE_UINT64:
+	  pspec = g_param_spec_uint64(name, nick, blurb,
+				      seed_value_to_uint64(ctx,
+							   jsmin_value,
+							   exception),
+				      seed_value_to_uint64(ctx,
+							   jsmax_value,
+							   exception),
+				      seed_value_to_uint64(ctx,
+							   jsdefault_value,
+							   exception), flags);
+	  break;
+	case G_TYPE_FLOAT:
+	  pspec = g_param_spec_float(name, nick, blurb,
+				     seed_value_to_float(ctx,
+							 jsmin_value,
+							 exception),
+				     seed_value_to_float(ctx,
+							 jsmax_value,
+							 exception),
+				     seed_value_to_float(ctx,
+							 jsdefault_value,
+							 exception), flags);
+	  break;
+	case G_TYPE_DOUBLE:
+	  pspec = g_param_spec_double(name, nick, blurb,
+				      seed_value_to_double(ctx,
+							   jsmin_value,
+							   exception),
+				      seed_value_to_double(ctx,
+							   jsmax_value,
+							   exception),
+				      seed_value_to_double(ctx,
+							   jsdefault_value,
+							   exception), flags);
+	  break;
+	  // TODO: support enums/flags/params
+	case G_TYPE_STRING:
+	  // TODO: leaky?
+	  pspec = g_param_spec_string(name, nick, blurb,
+				      seed_value_to_string(ctx,
+							   jsdefault_value,
+							   exception), flags);
+	  break;
+	case G_TYPE_OBJECT:
+	  jsobject_type = seed_object_get_property (ctx, property_def,
+						    "object_type");
+
+	  if (JSValueIsNull (ctx, jsobject_type) ||
+	      !JSValueIsNumber (ctx, jsobject_type))
+	    object_type = G_TYPE_NONE;
+	  else
+	    object_type = seed_value_to_long (ctx, jsobject_type, exception);
+
+	  if(object_type == G_TYPE_NONE)
+	    {
+	      seed_make_exception (ctx, exception, "PropertyInstallationError",
+				   "Property of type %s requires object_type attribute",
+				   g_type_name(property_type));
+	      return property_count;
+	    }
+
+	  pspec = g_param_spec_object(name, nick, blurb,
+				      object_type, flags);
+	  break;
+	case G_TYPE_BOXED: // Boxed types TODO: this is almost certainly wrong
+	  pspec = g_param_spec_boxed(name, nick, blurb, type, flags);
+	  break;
+	default:
+	case G_TYPE_NONE:
+	  seed_make_exception (ctx, exception, "PropertyInstallationError",
+			       "Property requires type attribute");
+	  return property_count;
+	  break;
+	}
+
+      g_object_class_install_property (g_class, ++property_count, pspec);
+
+      if (nick != name)
+	g_free (nick);
+
+      if (blurb != name)
+	g_free (blurb);
+
+      g_free (name);
+    }
+
+  return property_count;
 }
 
 static void
-seed_gtype_class_init (gpointer g_class,
-		       gpointer class_data)
+seed_gtype_class_init (gpointer g_class, gpointer class_data)
 {
   SeedGClassPrivates *priv;
   GIBaseInfo *class_info;
   JSContextRef ctx;
   JSValueRef jsargs[2];
   GType type;
-  JSValueRef exception = 0;
+  gchar *mes;
+  JSValueRef exception = NULL;
+  int initial_prop_count = 1;
 
-  priv = (SeedGClassPrivates *)class_data;
+  priv = (SeedGClassPrivates *) class_data;
 
   ((GObjectClass *)g_class)->get_property = seed_gtype_get_property;
   ((GObjectClass *)g_class)->set_property = seed_gtype_set_property;
@@ -438,24 +791,34 @@ seed_gtype_class_init (gpointer g_class,
   ctx = JSGlobalContextCreateInGroup (context_group, 0);
 
   type = (GType) JSObjectGetPrivate (priv->constructor);
-  seed_gtype_install_signals (ctx, priv->definition, type);
+  seed_gtype_install_signals (ctx, priv->definition, type, &exception);
+  initial_prop_count = seed_gtype_install_properties (ctx,
+						      priv->definition,
+						      type,
+						      (GObjectClass *) g_class,
+						      &exception);
 
   if (!priv->func)
     {
       JSGlobalContextRelease ((JSGlobalContextRef) ctx);
+      if (exception)
+	{
+	  mes = seed_exception_to_string (ctx, exception);
+	  g_warning ("Exception in class init closure. %s \n", mes);
+	  g_free (mes);
+	}
       return;
     }
 
-
   seed_prepare_global_context (ctx);
 
-
   class_info = seed_get_class_info_for_type (type);
 
   jsargs[0] = seed_make_struct (ctx, g_class, class_info);
   jsargs[1] = seed_gobject_get_prototype_for_gtype (type);
 
-  seed_attach_methods_to_class_object (ctx, (JSObjectRef) jsargs[0], &exception);
+  seed_attach_methods_to_class_object (ctx, (JSObjectRef) jsargs[0],
+				       &exception);
 
   g_base_info_unref ((GIBaseInfo *) class_info);
 
@@ -465,13 +828,14 @@ seed_gtype_class_init (gpointer g_class,
   seed_object_set_property (ctx, (JSObjectRef) jsargs[0],
 			    "type", seed_value_from_int (ctx, type, 0));
   seed_object_set_property (ctx, (JSObjectRef) jsargs[0],
-			    "property_count", seed_value_from_int (ctx, 1,
-								   0));
+			    "property_count",
+			    seed_value_from_int (ctx, initial_prop_count + 1,
+						 0));
 
   JSObjectCallAsFunction (ctx, priv->func, 0, 2, jsargs, &exception);
   if (exception)
     {
-      gchar *mes = seed_exception_to_string (ctx, exception);
+      mes = seed_exception_to_string (ctx, exception);
       g_warning ("Exception in class init closure. %s \n", mes);
       g_free (mes);
     }
@@ -479,12 +843,10 @@ seed_gtype_class_init (gpointer g_class,
   JSGlobalContextRelease ((JSGlobalContextRef) ctx);
 }
 
-
-
 static JSObjectRef
 seed_gtype_constructor_invoked (JSContextRef ctx,
 				JSObjectRef constructor,
-				size_t argumentCount,
+				gsize argumentCount,
 				const JSValueRef arguments[],
 				JSValueRef * exception)
 {
@@ -596,7 +958,7 @@ static JSValueRef
 seed_param_getter_invoked (JSContextRef ctx,
 			   JSObjectRef function,
 			   JSObjectRef thisObject,
-			   size_t argumentCount,
+			   gsize argumentCount,
 			   const JSValueRef arguments[],
 			   JSValueRef * exception)
 {
@@ -629,7 +991,7 @@ static JSValueRef
 seed_param_setter_invoked (JSContextRef ctx,
 			   JSObjectRef function,
 			   JSObjectRef thisObject,
-			   size_t argumentCount,
+			   gsize argumentCount,
 			   const JSValueRef arguments[],
 			   JSValueRef * exception)
 {
@@ -680,20 +1042,18 @@ seed_gtype_init (SeedEngine * local_eng)
 {
   JSClassDefinition gtype_def = kJSClassDefinitionEmpty;
 
-
   gtype_def.callAsConstructor = seed_gtype_constructor_invoked;
   seed_gtype_class = JSClassCreate (&gtype_def);
   JSClassRetain (seed_gtype_class);
 
-  seed_gtype_constructor = JSObjectMake (local_eng->context, seed_gtype_class, 0);
+  seed_gtype_constructor =
+    JSObjectMake (local_eng->context, seed_gtype_class, 0);
 
   seed_object_set_property (local_eng->context,
-			    local_eng->global, "GType", seed_gtype_constructor);
+			    local_eng->global, "GType",
+			    seed_gtype_constructor);
 
-  qgetter = g_quark_from_static_string ("js-getter");
-  qsetter = g_quark_from_static_string ("js-setter");
   qiinit = g_quark_from_static_string("js-instance-init");
-  qcinit = g_quark_from_static_string("js-class-init");
 
   seed_define_gtype_functions (local_eng->context);
 }
