@@ -1,3 +1,22 @@
+/* -*- mode: C; indent-tabs-mode: t; tab-width: 8; c-basic-offset: 2; -*- */
+
+/*
+ * This file is part of Seed, the GObject Introspection<->Javascript bindings.
+ *
+ * Seed is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation, either version 3 of
+ * the License, or (at your option) any later version.
+ * Seed is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with Seed.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Copyright (C) Robert Carr 2009 <carrr@rpi.edu>
+ */
+
 #define _GNU_SOURCE
 
 #include "../../config.h"
@@ -27,6 +46,28 @@ SeedObject os_namespace;
   seed_make_exception (ctx, exception, "ArgumentError", name " expected " argnum " got %Zd", argument_count); \
   return seed_make_undefined (ctx);
 
+SeedValue
+seed_os_realpath (SeedContext ctx,
+	          SeedObject function,
+	          SeedObject this_object,
+	          size_t argument_count,
+	          const SeedValue arguments[],
+	          SeedException * exception)
+{
+  gchar *arg;
+  gchar *ret;
+
+  if (argument_count != 1)
+    {
+      EXPECTED_EXCEPTION("os.realpath", "1 argument");
+    }
+  arg = seed_value_to_string (ctx, arguments[0], exception);
+  ret = canonicalize_file_name(arg);
+  g_free (arg);
+
+  return seed_value_from_string (ctx, ret, exception);
+}
+
 SeedValue
 seed_os_chdir (SeedContext ctx,
 	       SeedObject function,
@@ -981,6 +1022,7 @@ seed_os_fork (SeedContext ctx,
 
 seed_static_function os_funcs[] = {
   {"fork", seed_os_fork, 0},
+  {"realpath", seed_os_realpath, 0},
   {"chdir", seed_os_chdir, 0},
   {"fchdir", seed_os_fchdir, 0},
   {"getcwd", seed_os_getcwd, 0},
